@@ -2,12 +2,13 @@
 
 import itertools
 import multiprocessing as mp
+from warnings import warn
 
 import networkx as nx
 import numpy as np
 import pandas as pd
-from rtree.index import Index as RTreeIndex
 from shapely.geometry import Point
+from shapely.strtree import STRtree
 
 from . import projection
 from . import utils
@@ -101,7 +102,7 @@ def euclidean_dist_vec(y1, x1, y2, x2):
     return ((x1 - x2) ** 2 + (y1 - y2) ** 2) ** 0.5
 
 
-def add_edge_lengths(G, precision=3, edges=None):
+def add_edge_lengths(G, precision=None, edges=None):
     """
     Add `length` attribute (in meters) to each edge.
 
@@ -126,7 +127,7 @@ def add_edge_lengths(G, precision=3, edges=None):
     G : networkx.MultiDiGraph
         unprojected, unsimplified input graph
     precision : int
-        decimal precision to round lengths
+        deprecated, do not use
     edges : tuple
         tuple of (u, v, k) tuples representing subset of edges to add length
         attributes to. if None, add lengths to all edges.
@@ -136,6 +137,14 @@ def add_edge_lengths(G, precision=3, edges=None):
     G : networkx.MultiDiGraph
         graph with edge length attributes
     """
+    if precision is None:
+        precision = 3
+    else:
+        warn(
+            "the `precision` parameter is deprecated and will be removed in a future release",
+            stacklevel=2,
+        )
+
     if edges is None:
         uvk = tuple(G.edges)
     else:
@@ -147,8 +156,12 @@ def add_edge_lengths(G, precision=3, edges=None):
     try:
         # two-dimensional array of coordinates: y0, x0, y1, x1
         c = np.array([(y[u], x[u], y[v], x[v]) for u, v, k in uvk])
-    except KeyError:  # pragma: no cover
-        raise KeyError("some edges missing nodes, possibly due to input data clipping issue")
+        # ensure all coordinates can be converted to float and are non-null
+        assert not np.isnan(c.astype(float)).any()
+    except (AssertionError, KeyError) as e:  # pragma: no cover
+        raise ValueError(
+            "some edges missing nodes, possibly due to input data clipping issue"
+        ) from e
 
     # calculate great circle distances, round, and fill nulls with zeros
     dists = great_circle_vec(c[:, 0], c[:, 1], c[:, 2], c[:, 3]).round(precision)
@@ -240,21 +253,9 @@ def nearest_edges(G, X, Y, interpolate=None, return_dist=False):
 
     If `X` and `Y` are single coordinate values, this will return the nearest
     edge to that point. If `X` and `Y` are lists of coordinate values, this
-    will return the nearest edge to each point.
-
-    If `interpolate` is None, search for the nearest edge to each point, one
-    at a time, using an r-tree and minimizing the euclidean distances from the
-    point to the possible matches. For accuracy, use a projected graph and
-    points. This method is precise and also fastest if searching for few
-    points relative to the graph's size.
-
-    For a faster method if searching for many points relative to the graph's
-    size, use the `interpolate` argument to interpolate points along the edges
-    and index them. If the graph is projected, this uses a k-d tree for
-    euclidean nearest neighbor search, which requires that scipy is installed
-    as an optional dependency. If graph is unprojected, this uses a ball tree
-    for haversine nearest neighbor search, which requires that scikit-learn is
-    installed as an optional dependency.
+    will return the nearest edge to each point. This function uses an R-tree
+    spatial index and minimizes the euclidean distance from each point to the
+    possible matches. For accurate results, use a projected graph and points.
 
     Parameters
     ----------
@@ -267,8 +268,7 @@ def nearest_edges(G, X, Y, interpolate=None, return_dist=False):
         points' y (latitude) coordinates, in same CRS/units as graph and
         containing no nulls
     interpolate : float
-        spacing distance between interpolated points, in same units as graph.
-        smaller values generate more points.
+        deprecated, do not use
     return_dist : bool
         optionally also return distance between points and nearest edges
 
@@ -291,25 +291,27 @@ def nearest_edges(G, X, Y, interpolate=None, return_dist=False):
 
     # if no interpolation distance was provided
     if interpolate is None:
+        # build an r-tree spatial index by position for subsequent iloc
+        rtree = STRtree(geoms)
 
-        # build the r-tree spatial index by position for subsequent iloc
-        rtree = RTreeIndex()
-        for pos, bounds in enumerate(geoms.bounds.values):
-            rtree.insert(pos, bounds)
+        # use the r-tree to find each point's nearest neighbor and distance
+        points = [Point(xy) for xy in zip(X, Y)]
+        pos, dist = rtree.query_nearest(points, all_matches=False, return_distance=True)
 
-        # use r-tree to find possible nearest neighbors, one point at a time,
-        # then minimize euclidean distance from point to the possible matches
-        ne_dist = list()
-        for xy in zip(X, Y):
-            dists = geoms.iloc[list(rtree.nearest(xy, num_results=10))].distance(Point(xy))
-            ne_dist.append((dists.idxmin(), dists.min()))
-        ne, dist = zip(*ne_dist)
+        # if user passed X/Y lists, the 2nd subarray contains geom indices
+        if len(pos.shape) > 1:
+            pos = pos[1]
+        ne = geoms.iloc[pos].index
 
     # otherwise, if interpolation distance was provided
     else:
+        warn(
+            "The `interpolate` parameter has been deprecated and will be removed in a future release",
+            stacklevel=2,
+        )
 
         # interpolate points along edges to index with k-d tree or ball tree
-        uvk_xy = list()
+        uvk_xy = []
         for uvk, geom in zip(geoms.index, geoms.values):
             uvk_xy.extend((uvk, xy) for xy in utils_geo.interpolate_points(geom, interpolate))
         labels, xy = zip(*uvk_xy)
@@ -351,7 +353,7 @@ def _single_shortest_path(G, orig, dest, weight):
     Solve the shortest path from an origin node to a destination node.
 
     This function is a convenience wrapper around networkx.shortest_path, with
-    exception handling for unsolvable paths.
+    exception handling for unsolvable paths. It uses Dijkstra's algorithm.
 
     Parameters
     ----------
@@ -370,7 +372,7 @@ def _single_shortest_path(G, orig, dest, weight):
         list of node IDs constituting the shortest path
     """
     try:
-        return nx.shortest_path(G, orig, dest, weight=weight)
+        return nx.shortest_path(G, orig, dest, weight=weight, method="dijkstra")
     except nx.exception.NetworkXNoPath:  # pragma: no cover
         utils.log(f"Cannot solve path from {orig} to {dest}")
         return None
@@ -380,13 +382,13 @@ def shortest_path(G, orig, dest, weight="length", cpus=1):
     """
     Solve shortest path from origin node(s) to destination node(s).
 
-    If `orig` and `dest` are single node IDs, this will return a list of the
-    nodes constituting the shortest path between them.  If `orig` and `dest`
-    are lists of node IDs, this will return a list of lists of the nodes
-    constituting the shortest path between each origin-destination pair. If a
-    path cannot be solved, this will return None for that path. You can
-    parallelize solving multiple paths with the `cpus` parameter, but be
-    careful to not exceed your available RAM.
+    Uses Dijkstra's algorithm. If `orig` and `dest` are single node IDs, this
+    will return a list of the nodes constituting the shortest path between
+    them. If `orig` and `dest` are lists of node IDs, this will return a list
+    of lists of the nodes constituting the shortest path between each
+    origin-destination pair. If a path cannot be solved, this will return None
+    for that path. You can parallelize solving multiple paths with the `cpus`
+    parameter, but be careful to not exceed your available RAM.
 
     See also `k_shortest_paths` to solve multiple shortest paths between a
     single origin and destination. For additional functionality or different
@@ -411,12 +413,14 @@ def shortest_path(G, orig, dest, weight="length", cpus=1):
         list of node IDs constituting the shortest path, or, if orig and dest
         are lists, then a list of path lists
     """
+    _verify_edge_attribute(G, weight)
+
+    # if neither orig nor dest is iterable, just return the shortest path
     if not (hasattr(orig, "__iter__") or hasattr(dest, "__iter__")):
-        # if neither orig nor dest is iterable, just return the shortest path
         return _single_shortest_path(G, orig, dest, weight)
 
+    # if both orig and dest are iterables, ensure they have same lengths
     elif hasattr(orig, "__iter__") and hasattr(dest, "__iter__"):
-        # if both orig and dest are iterables ensure they have same lengths
         if len(orig) != len(dest):  # pragma: no cover
             raise ValueError("orig and dest must contain same number of elements")
 
@@ -425,11 +429,12 @@ def shortest_path(G, orig, dest, weight="length", cpus=1):
         cpus = min(cpus, mp.cpu_count())
         utils.log(f"Solving {len(orig)} paths with {cpus} CPUs...")
 
+        # if single-threading, calculate each shortest path one at a time
         if cpus == 1:
-            # if single-threading, calculate each shortest path one at a time
             paths = [_single_shortest_path(G, o, d, weight) for o, d in zip(orig, dest)]
+
+        # if multi-threading, calculate shortest paths in parallel
         else:
-            # if multi-threading, calculate shortest paths in parallel
             args = ((G, o, d, weight) for o, d in zip(orig, dest))
             pool = mp.Pool(cpus)
             sma = pool.starmap_async(_single_shortest_path, args)
@@ -439,8 +444,8 @@ def shortest_path(G, orig, dest, weight="length", cpus=1):
 
         return paths
 
+    # if only one of orig or dest is iterable and the other is not
     else:  # pragma: no cover
-        # if only one of orig or dest is iterable and the other is not
         raise ValueError("orig and dest must either both be iterable or neither must be iterable")
 
 
@@ -448,7 +453,8 @@ def k_shortest_paths(G, orig, dest, k, weight="length"):
     """
     Solve `k` shortest paths from an origin node to a destination node.
 
-    See also `shortest_path` to get just the one shortest path.
+    Uses Yen's algorithm. See also `shortest_path` to solve just the one
+    shortest path.
 
     Parameters
     ----------
@@ -470,6 +476,34 @@ def k_shortest_paths(G, orig, dest, k, weight="length"):
         a generator of `k` shortest paths ordered by total weight. each path
         is a list of node IDs.
     """
+    _verify_edge_attribute(G, weight)
     paths_gen = nx.shortest_simple_paths(utils_graph.get_digraph(G, weight), orig, dest, weight)
     for path in itertools.islice(paths_gen, 0, k):
         yield path
+
+
+def _verify_edge_attribute(G, attr):
+    """
+    Verify attribute values are numeric and non-null across graph edges.
+
+    Raises a `ValueError` if attribute contains non-numeric values and raises
+    a warning if attribute is missing or null on any edges.
+
+    Parameters
+    ----------
+    G : networkx.MultiDiGraph
+        input graph
+    attr : string
+        edge attribute to verify
+
+    Returns
+    -------
+    None
+    """
+    try:
+        values = np.array(tuple(G.edges(data=attr)))[:, 2]
+        values_float = values.astype(float)
+        if np.isnan(values_float).any():
+            warn(f"The attribute {attr!r} is missing or null on some edges.", stacklevel=2)
+    except ValueError as e:
+        raise ValueError(f"The edge attribute {attr!r} contains non-numeric values.") from e
