@@ -4,13 +4,14 @@ import multiprocessing as mp
 import time
 from hashlib import sha1
 from pathlib import Path
+from warnings import warn
 
 import networkx as nx
 import numpy as np
 import pandas as pd
 import requests
 
-from . import downloader
+from . import _downloader
 from . import utils
 from . import utils_graph
 
@@ -109,21 +110,30 @@ def add_node_elevations_raster(G, filepath, band=1, cpus=None):
 
 
 def add_node_elevations_google(
-    G, api_key, max_locations_per_batch=350, pause_duration=0, precision=3
+    G,
+    api_key=None,
+    max_locations_per_batch=350,
+    pause_duration=0,
+    precision=None,
+    url_template="https://maps.googleapis.com/maps/api/elevation/json?locations={}&key={}",
 ):  # pragma: no cover
     """
     Add `elevation` (meters) attribute to each node using a web service.
 
-    This uses the Google Maps Elevation API and requires an API key. For a
-    free, local alternative, see the `add_node_elevations_raster` function.
-    See also the `add_edge_grades` function.
+    By default, this uses the Google Maps Elevation API but you can optionally
+    use an equivalent API with the same interface and response format, such as
+    Open Topo Data. The Google Maps Elevation API requires an API key but
+    other providers may not.
+
+    For a free local alternative see the `add_node_elevations_raster`
+    function. See also the `add_edge_grades` function.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         input graph
     api_key : string
-        a Google Maps Elevation API key
+        a valid API key, сan be None if the API does not require a key
     max_locations_per_batch : int
         max number of coordinate pairs to submit in each API call (if this is
         too high, the server will reject the request because its character
@@ -132,15 +142,24 @@ def add_node_elevations_google(
         time to pause between API calls, which can be increased if you get
         rate limited
     precision : int
-        decimal precision to round elevation values
+        deprecated, do not use
+    url_template : string
+        a URL string template for the API endpoint, containing exactly two
+        parameters: `locations` and `key`; for example, for Open Topo Data:
+        `"https://api.opentopodata.org/v1/aster30m?locations={}&key={}"`
 
     Returns
     -------
     G : networkx.MultiDiGraph
         graph with node elevation attributes
     """
-    # elevation API endpoint ready for use
-    url_template = "https://maps.googleapis.com/maps/api/elevation/json?locations={}&key={}"
+    if precision is None:
+        precision = 3
+    else:
+        warn(
+            "the `precision` parameter is deprecated and will be removed in a future release",
+            stacklevel=2,
+        )
 
     # make a pandas series of all the nodes' coordinates as 'lat,lng'
     # round coordinates to 5 decimal places (approx 1 meter) to be able to fit
@@ -160,20 +179,21 @@ def add_node_elevations_google(
         url = url_template.format(locations, api_key)
 
         # check if this request is already in the cache (if global use_cache=True)
-        cached_response_json = downloader._retrieve_from_cache(url)
+        cached_response_json = _downloader._retrieve_from_cache(url)
         if cached_response_json is not None:
             response_json = cached_response_json
         else:
-            try:
-                # request the elevations from the API
-                utils.log(f"Requesting node elevations: {url}")
-                time.sleep(pause_duration)
-                response = requests.get(url)
+            # request the elevations from the API
+            utils.log(f"Requesting node elevations: {url}")
+            time.sleep(pause_duration)
+            response = requests.get(url)
+            if response.status_code == 200:
                 response_json = response.json()
-                downloader._save_to_cache(url, response_json, response.status_code)
-            except Exception as e:
-                utils.log(e)
-                utils.log(f"Server responded with {response.status_code}: {response.reason}")
+                _downloader._save_to_cache(url, response_json, response.status_code)
+            else:
+                raise Exception(
+                    f"Server responded with {response.status_code}: {response.reason} \n{response.json()}"
+                )
 
         # append these elevation results to the list of all results
         results.extend(response_json["results"])
@@ -181,24 +201,22 @@ def add_node_elevations_google(
     # sanity check that all our vectors have the same number of elements
     if not (len(results) == len(G) == len(node_points)):
         raise Exception(
-            f"Graph has {len(G)} nodes but we received {len(results)} results from elevation API"
+            f"Graph has {len(G)} nodes but we received {len(results)} results. \n{response_json}"
         )
     else:
-        utils.log(
-            f"Graph has {len(G)} nodes and we received {len(results)} results from elevation API"
-        )
+        utils.log(f"Graph has {len(G)} nodes and we received {len(results)} results.")
 
     # add elevation as an attribute to the nodes
     df = pd.DataFrame(node_points, columns=["node_points"])
     df["elevation"] = [result["elevation"] for result in results]
     df["elevation"] = df["elevation"].round(precision)
     nx.set_node_attributes(G, name="elevation", values=df["elevation"].to_dict())
-    utils.log("Added elevation data from Google to all nodes.")
+    utils.log("Added elevation data from API to all nodes.")
 
     return G
 
 
-def add_edge_grades(G, add_absolute=True, precision=3):
+def add_edge_grades(G, add_absolute=True, precision=None):
     """
     Add `grade` attribute to each graph edge.
 
@@ -216,13 +234,21 @@ def add_edge_grades(G, add_absolute=True, precision=3):
     add_absolute : bool
         if True, also add absolute value of grade as `grade_abs` attribute
     precision : int
-        decimal precision to round grade values
+        deprecated, do not use
 
     Returns
     -------
     G : networkx.MultiDiGraph
         graph with edge `grade` (and optionally `grade_abs`) attributes
     """
+    if precision is None:
+        precision = 3
+    else:
+        warn(
+            "the `precision` parameter is deprecated and will be removed in a future release",
+            stacklevel=2,
+        )
+
     elev_lookup = G.nodes(data="elevation")
     u, v, k, lengths = zip(*G.edges(keys=True, data="length"))
     uvk = tuple(zip(u, v, k))
