@@ -1,4 +1,10 @@
-"""Geocode queries and create GeoDataFrames of place boundaries."""
+"""
+Geocode place names or addresses or retrieve OSM elements by place name or ID.
+
+This module uses the Nominatim API's "search" and "lookup" endpoints. For more
+details see https://wiki.openstreetmap.org/wiki/Elements and
+https://nominatim.org/.
+"""
 
 import logging as lg
 from collections import OrderedDict
@@ -6,7 +12,7 @@ from collections import OrderedDict
 import geopandas as gpd
 import pandas as pd
 
-from . import downloader
+from . import _downloader
 from . import projection
 from . import settings
 from . import utils
@@ -14,7 +20,9 @@ from . import utils
 
 def geocode(query):
     """
-    Geocode a query string to (lat, lng) with the Nominatim geocoder.
+    Geocode place names or addresses to (lat, lng) with the Nominatim API.
+
+    This geocodes the query via the Nominatim "search" endpoint.
 
     Parameters
     ----------
@@ -32,48 +40,52 @@ def geocode(query):
     params["limit"] = 1
     params["dedupe"] = 0  # prevent deduping to get precise number of results
     params["q"] = query
-    response_json = downloader.nominatim_request(params=params)
+    response_json = _downloader._nominatim_request(params=params)
 
     # if results were returned, parse lat and lng out of the result
     if response_json and "lat" in response_json[0] and "lon" in response_json[0]:
         lat = float(response_json[0]["lat"])
         lng = float(response_json[0]["lon"])
         point = (lat, lng)
-        utils.log(f'Geocoded "{query}" to {point}')
+        utils.log(f"Geocoded {query!r} to {point}")
         return point
     else:
-        raise ValueError(f'Nominatim could not geocode query "{query}"')
+        raise ValueError(f"Nominatim could not geocode query {query!r}")
 
 
 def geocode_to_gdf(query, which_result=None, by_osmid=False, buffer_dist=None):
     """
-    Retrieve place(s) by name or ID from the Nominatim API as a GeoDataFrame.
+    Retrieve OSM elements by place name or OSM ID with the Nominatim API.
 
-    You can query by place name or OSM ID. If querying by place name, the
-    query argument can be a string or structured dict, or a list of such
-    strings/dicts to send to geocoder. You can instead query by OSM ID by
-    setting `by_osmid=True`. In this case, geocode_to_gdf treats the query
-    argument as an OSM ID (or list of OSM IDs) for Nominatim lookup rather
-    than text search. OSM IDs must be prepended with their types: node (N),
-    way (W), or relation (R), in accordance with the Nominatim format. For
-    example, `query=["R2192363", "N240109189", "W427818536"]`.
+    If searching by place name, the `query` argument can be a string or
+    structured dict, or a list of such strings/dicts to send to the geocoder.
+    This uses the Nominatim "search" endpoint to geocode the place name to the
+    best-matching OSM element, then returns that element and its attribute
+    data.
 
-    If query argument is a list, then which_result should be either a single
-    value or a list with the same length as query. The queries you provide
-    must be resolvable to places in the Nominatim database. The resulting
-    GeoDataFrame's geometry column contains place boundaries if they exist in
-    OpenStreetMap.
+    You can instead query by OSM ID by passing `by_osmid=True`. This uses the
+    Nominatim "lookup" endpoint to retrieve the OSM element with that ID. In
+    this case, the function treats the `query` argument as an OSM ID (or list
+    of OSM IDs), which must be prepended with their types: node (N), way (W),
+    or relation (R) in accordance with the Nominatim API format. For example,
+    `query=["R2192363", "N240109189", "W427818536"]`.
+
+    If `query` is a list, then `which_result` must be either a single value or
+    a list with the same length as `query`. The queries you provide must be
+    resolvable to elements in the Nominatim database. The resulting
+    GeoDataFrame's geometry column contains place boundaries if they exist.
 
     Parameters
     ----------
-    query : string or dict or list
+    query : string or dict or list of strings/dicts
         query string(s) or structured dict(s) to geocode
     which_result : int
-        which geocoding result to use. if None, auto-select the first
+        which search result to return. if None, auto-select the first
         (Multi)Polygon or raise an error if OSM doesn't return one. to get
-        the top match regardless of geometry type, set which_result=1
+        the top match regardless of geometry type, set which_result=1.
+        ignored if by_osmid=True.
     by_osmid : bool
-        if True, handle query as an OSM ID for lookup rather than text search
+        if True, treat query as an OSM ID lookup rather than text search
     buffer_dist : float
         distance to buffer around the place geometry, in meters
 
@@ -112,7 +124,7 @@ def geocode_to_gdf(query, which_result=None, by_osmid=False, buffer_dist=None):
 
     # reset GeoDataFrame index and set its CRS
     gdf = gdf.reset_index(drop=True)
-    gdf.crs = settings.default_crs
+    gdf = gdf.set_crs(settings.default_crs)
 
     # if buffer_dist was passed in, project the geometry to UTM, buffer it in
     # meters, then project it back to lat-lng
@@ -137,7 +149,8 @@ def _geocode_query_to_gdf(query, which_result, by_osmid):
     which_result : int
         which geocoding result to use. if None, auto-select the first
         (Multi)Polygon or raise an error if OSM doesn't return one. to get
-        the top match regardless of geometry type, set which_result=1
+        the top match regardless of geometry type, set which_result=1.
+        ignored if by_osmid=True.
     by_osmid : bool
         if True, handle query as an OSM ID for lookup rather than text search
 
@@ -151,12 +164,12 @@ def _geocode_query_to_gdf(query, which_result, by_osmid):
     else:
         limit = which_result
 
-    results = downloader._osm_place_download(query, by_osmid=by_osmid, limit=limit)
+    results = _downloader._retrieve_osm_element(query, by_osmid=by_osmid, limit=limit)
 
     # choose the right result from the JSON response
     if not results:
         # if no results were returned, raise error
-        raise ValueError(f'Nominatim geocoder returned 0 results for query "{query}"')
+        raise ValueError(f"Nominatim geocoder returned 0 results for query {query!r}")
 
     elif by_osmid:
         # if searching by OSM ID, always take the first (ie, only) result
@@ -172,13 +185,13 @@ def _geocode_query_to_gdf(query, which_result, by_osmid):
 
     else:  # pragma: no cover
         # else, we got fewer results than which_result, raise error
-        msg = f'Nominatim geocoder only returned {len(results)} result(s) for query "{query}"'
+        msg = f"Nominatim geocoder only returned {len(results)} result(s) for query {query!r}"
         raise ValueError(msg)
 
     # if we got a non (Multi)Polygon geometry type (like a point), log warning
     geom_type = result["geojson"]["type"]
     if geom_type not in {"Polygon", "MultiPolygon"}:
-        msg = f'Nominatim geocoder returned a {geom_type} as the geometry for query "{query}"'
+        msg = f"Nominatim geocoder returned a {geom_type} as the geometry for query {query!r}"
         utils.log(msg, level=lg.WARNING)
 
     # build the GeoJSON feature from the chosen result
@@ -213,7 +226,7 @@ def _get_first_polygon(results, query):
     Parameters
     ----------
     results : list
-        list of results from downloader._osm_place_download
+        list of results from _downloader._osm_place_download
     query : str
         the query string or structured dict that was geocoded
 
@@ -228,4 +241,4 @@ def _get_first_polygon(results, query):
             return result
 
     # if we never found a polygon, throw an error
-    raise ValueError(f'Nominatim could not geocode query "{query}" to polygonal boundaries')
+    raise ValueError(f"Nominatim could not geocode query {query!r} to polygonal boundaries")
