@@ -1,14 +1,14 @@
-"""Graph creation functions."""
+"""Download and create graphs from OpenStreetMap data."""
 
 import itertools
-import warnings
+from warnings import warn
 
 import networkx as nx
 from shapely.geometry import MultiPolygon
 from shapely.geometry import Polygon
 
+from . import _downloader
 from . import distance
-from . import downloader
 from . import geocoder
 from . import osm_xml
 from . import projection
@@ -36,7 +36,11 @@ def graph_from_bbox(
     custom_filter=None,
 ):
     """
-    Create a graph from OSM within some bounding box.
+    Download and create a graph within some bounding box.
+
+    You can use the `settings` module to retrieve a snapshot of historical OSM
+    data as of a certain date, or to configure the Overpass server timeout,
+    memory allocation, and other custom settings.
 
     Parameters
     ----------
@@ -73,10 +77,9 @@ def graph_from_bbox(
 
     Notes
     -----
-    You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via the `settings` module. Very large query areas
-    will use the utils_geo._consolidate_subdivide_geometry function to perform
-    multiple queries: see that function's documentation for caveats.
+    Very large query areas use the `utils_geo._consolidate_subdivide_geometry`
+    function to automatically make multiple requests: see that function's
+    documentation for caveats.
     """
     # convert bounding box to a polygon
     polygon = utils_geo.bbox_to_poly(north, south, east, west)
@@ -108,7 +111,11 @@ def graph_from_point(
     custom_filter=None,
 ):
     """
-    Create a graph from OSM within some distance of some (lat, lng) point.
+    Download and create a graph within some distance of a (lat, lng) point.
+
+    You can use the `settings` module to retrieve a snapshot of historical OSM
+    data as of a certain date, or to configure the Overpass server timeout,
+    memory allocation, and other custom settings.
 
     Parameters
     ----------
@@ -120,8 +127,7 @@ def graph_from_point(
     dist_type : string {"network", "bbox"}
         if "bbox", retain only those nodes within a bounding box of the
         distance parameter. if "network", retain only those nodes within some
-        network distance from the center-most node (requires that scikit-learn
-        is installed as an optional dependency).
+        network distance from the center-most node.
     network_type : string, {"all_private", "all", "bike", "drive", "drive_service", "walk"}
         what type of street network to get if custom_filter is None
     simplify : bool
@@ -147,10 +153,9 @@ def graph_from_point(
 
     Notes
     -----
-    You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via the `settings` module. Very large query areas
-    will use the utils_geo._consolidate_subdivide_geometry function to perform
-    multiple queries: see that function's documentation for caveats.
+    Very large query areas use the `utils_geo._consolidate_subdivide_geometry`
+    function to automatically make multiple requests: see that function's
+    documentation for caveats.
     """
     if dist_type not in {"bbox", "network"}:  # pragma: no cover
         raise ValueError('dist_type must be "bbox" or "network"')
@@ -195,7 +200,11 @@ def graph_from_address(
     custom_filter=None,
 ):
     """
-    Create a graph from OSM within some distance of some address.
+    Download and create a graph within some distance of an address.
+
+    You can use the `settings` module to retrieve a snapshot of historical OSM
+    data as of a certain date, or to configure the Overpass server timeout,
+    memory allocation, and other custom settings.
 
     Parameters
     ----------
@@ -208,8 +217,7 @@ def graph_from_address(
     dist_type : string {"network", "bbox"}
         if "bbox", retain only those nodes within a bounding box of the
         distance parameter. if "network", retain only those nodes within some
-        network distance from the center-most node (requires that scikit-learn
-        is installed as an optional dependency).
+        network distance from the center-most node.
     network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
         what type of street network to get if custom_filter is None
     simplify : bool
@@ -237,10 +245,9 @@ def graph_from_address(
 
     Notes
     -----
-    You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via the `settings` module. Very large query areas
-    will use the utils_geo._consolidate_subdivide_geometry function to perform
-    multiple queries: see that function's documentation for caveats.
+    Very large query areas use the `utils_geo._consolidate_subdivide_geometry`
+    function to automatically make multiple requests: see that function's
+    documentation for caveats.
     """
     # geocode the address string to a (lat, lng) point
     point = geocoder.geocode(query=address)
@@ -277,7 +284,7 @@ def graph_from_place(
     custom_filter=None,
 ):
     """
-    Create graph from OSM within the boundaries of some geocodable place(s).
+    Download and create a graph within the boundaries of some place(s).
 
     The query must be geocodable and OSM must have polygon boundaries for the
     geocode result. If OSM does not have a polygon for this place, you can
@@ -291,6 +298,10 @@ def graph_from_place(
     the OSM ID of the place, you can retrieve its boundary polygon using the
     geocode_to_gdf function, then pass it to the graph_from_polygon function.
 
+    You can use the `settings` module to retrieve a snapshot of historical OSM
+    data as of a certain date, or to configure the Overpass server timeout,
+    memory allocation, and other custom settings.
+
     Parameters
     ----------
     query : string or dict or list
@@ -325,10 +336,9 @@ def graph_from_place(
 
     Notes
     -----
-    You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via the `settings` module. Very large query areas
-    will use the utils_geo._consolidate_subdivide_geometry function to perform
-    multiple queries: see that function's documentation for caveats.
+    Very large query areas use the `utils_geo._consolidate_subdivide_geometry`
+    function to automatically make multiple requests: see that function's
+    documentation for caveats.
     """
     # create a GeoDataFrame with the spatial boundaries of the place(s)
     if isinstance(query, (str, dict)):
@@ -372,7 +382,11 @@ def graph_from_polygon(
     custom_filter=None,
 ):
     """
-    Create a graph from OSM within the boundaries of some shapely polygon.
+    Download and create a graph within the boundaries of a (multi)polygon.
+
+    You can use the `settings` module to retrieve a snapshot of historical OSM
+    data as of a certain date, or to configure the Overpass server timeout,
+    memory allocation, and other custom settings.
 
     Parameters
     ----------
@@ -404,10 +418,9 @@ def graph_from_polygon(
 
     Notes
     -----
-    You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via the `settings` module. Very large query areas
-    will use the utils_geo._consolidate_subdivide_geometry function to perform
-    multiple queries: see that function's documentation for caveats.
+    Very large query areas use the `utils_geo._consolidate_subdivide_geometry`
+    function to automatically make multiple requests: see that function's
+    documentation for caveats.
     """
     # verify that the geometry is valid and is a shapely Polygon/MultiPolygon
     # before proceeding
@@ -429,7 +442,7 @@ def graph_from_polygon(
         poly_buff, _ = projection.project_geometry(poly_proj_buff, crs=crs_utm, to_latlong=True)
 
         # download the network data from OSM within buffered polygon
-        response_jsons = downloader._osm_network_download(poly_buff, network_type, custom_filter)
+        response_jsons = _downloader._osm_network_download(poly_buff, network_type, custom_filter)
 
         # create buffered graph from the downloaded data
         bidirectional = network_type in settings.bidirectional_network_types
@@ -461,7 +474,7 @@ def graph_from_polygon(
     # if clean_periphery=False, just use the polygon as provided
     else:
         # download the network data from OSM
-        response_jsons = downloader._osm_network_download(polygon, network_type, custom_filter)
+        response_jsons = _downloader._osm_network_download(polygon, network_type, custom_filter)
 
         # create graph from the downloaded data
         bidirectional = network_type in settings.bidirectional_network_types
@@ -481,11 +494,11 @@ def graph_from_polygon(
         # it's best to parameterize function with clean_periphery=True
         spn = stats.count_streets_per_node(G)
         nx.set_node_attributes(G, values=spn, name="street_count")
-        msg = (
+        warn(
             "the graph-level street_count attribute will likely be inaccurate "
-            "when you set clean_periphery=False"
+            "when you set clean_periphery=False",
+            stacklevel=2,
         )
-        warnings.warn(msg)
 
     utils.log(f"graph_from_polygon returned graph with {len(G)} nodes and {len(G.edges)} edges")
     return G
@@ -551,7 +564,8 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
 
     # make sure we got data back from the server request(s)
     if not any(rj["elements"] for rj in response_jsons):  # pragma: no cover
-        raise EmptyOverpassResponse("There are no data elements in the response JSON")
+        msg = "There are no data elements in the server response. Check log and query location/filters."
+        raise EmptyOverpassResponse(msg)
 
     # create the graph as a MultiDiGraph and set its meta-attributes
     metadata = {
@@ -562,8 +576,8 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
     G = nx.MultiDiGraph(**metadata)
 
     # extract nodes and paths from the downloaded osm data
-    nodes = dict()
-    paths = dict()
+    nodes = {}
+    paths = {}
     for response_json in response_jsons:
         nodes_temp, paths_temp = _parse_nodes_paths(response_json)
         nodes.update(nodes_temp)
@@ -649,8 +663,8 @@ def _parse_nodes_paths(response_json):
     nodes, paths : tuple of dicts
         dicts' keys = osmid and values = dict of attributes
     """
-    nodes = dict()
-    paths = dict()
+    nodes = {}
+    paths = {}
     for element in response_json["elements"]:
         if element["type"] == "node":
             nodes[element["id"]] = _convert_node(element)
@@ -667,7 +681,7 @@ def _is_path_one_way(path, bidirectional, oneway_values):
     Parameters
     ----------
     path : dict
-        a path's tag:value attribute data
+        a path's `tag:value` attribute data
     bidirectional : bool
         whether this is a bi-directional network type
     oneway_values : set
@@ -715,7 +729,7 @@ def _is_path_reversed(path, reversed_values):
     Parameters
     ----------
     path : dict
-        a path's tag:value attribute data
+        a path's `tag:value` attribute data
     reversed_values : set
         the values OSM uses in its 'oneway' tag to denote travel can only
         occur in the opposite direction of the node order
@@ -739,7 +753,7 @@ def _add_paths(G, paths, bidirectional=False):
     G : networkx.MultiDiGraph
         graph to add paths to
     paths : list
-        list of paths' tag:value attribute data dicts
+        list of paths' `tag:value` attribute data dicts
     bidirectional : bool
         if True, create bi-directional edges for one-way streets
 
@@ -755,7 +769,6 @@ def _add_paths(G, paths, bidirectional=False):
     reversed_values = {"-1", "reverse", "T"}
 
     for path in paths:
-
         # extract/remove the ordered list of nodes from this path element so
         # we don't add it as a superfluous attribute to the edge later
         nodes = path.pop("nodes")
