@@ -179,7 +179,7 @@ def _get_paths_to_simplify(G, strict=True):
     path_to_simplify : list
     """
     # first identify all the nodes that are endpoints
-    endpoints = set([n for n in G.nodes if _is_endpoint(G, n, strict=strict)])
+    endpoints = {n for n in G.nodes if _is_endpoint(G, n, strict=strict)}
     utils.log(f"Identified {len(endpoints)} edge endpoints")
 
     # for each endpoint node, look at each of its successor nodes
@@ -192,7 +192,32 @@ def _get_paths_to_simplify(G, strict=True):
                 yield _build_path(G, endpoint, successor, endpoints)
 
 
-def simplify_graph(G, strict=True, remove_rings=True):
+def _remove_rings(G):
+    """
+    Remove self-contained rings from graph.
+
+    This identifies any connected components that form a self-contained ring
+    without any endpoints, and removes them from the graph.
+
+    Parameters
+    ----------
+    G : networkx.MultiDiGraph
+        input graph
+
+    Returns
+    -------
+    G : networkx.MultiDiGraph
+        graph with self-contained rings removed
+    """
+    nodes_in_rings = set()
+    for wcc in nx.weakly_connected_components(G):
+        if not any(_is_endpoint(G, n) for n in wcc):
+            nodes_in_rings.update(wcc)
+    G.remove_nodes_from(nodes_in_rings)
+    return G
+
+
+def simplify_graph(G, strict=True, remove_rings=True, track_merged=False):
     """
     Simplify a graph's topology by removing interstitial nodes.
 
@@ -202,7 +227,9 @@ def simplify_graph(G, strict=True, remove_rings=True):
     a new `geometry` attribute on the new edge. Note that only simplified
     edges receive a `geometry` attribute. Some of the resulting consolidated
     edges may comprise multiple OSM ways, and if so, their multiple attribute
-    values are stored as a list.
+    values are stored as a list. Optionally, the simplified edges can receive
+    a `merged_edges` attribute that contains a list of all the (u, v) node
+    pairs that were merged together.
 
     Parameters
     ----------
@@ -215,6 +242,9 @@ def simplify_graph(G, strict=True, remove_rings=True):
         have multiple OSM IDs within them too.
     remove_rings : bool
         if True, remove isolated self-contained rings that have no endpoints
+    track_merged : bool
+        if True, add `merged_edges` attribute on simplified edges, containing
+        a list of all the (u, v) node pairs that were merged together
 
     Returns
     -------
@@ -239,11 +269,14 @@ def simplify_graph(G, strict=True, remove_rings=True):
 
     # generate each path that needs to be simplified
     for path in _get_paths_to_simplify(G, strict=strict):
-
         # add the interstitial edges we're removing to a list so we can retain
         # their spatial geometry
-        path_attributes = dict()
+        merged_edges = []
+        path_attributes = {}
         for u, v in zip(path[:-1], path[1:]):
+            if track_merged:
+                # keep track of the edges that were merged
+                merged_edges.append((u, v))
 
             # there should rarely be multiple edges between interstitial nodes
             # usually happens if OSM has duplicate ways digitized for just one
@@ -254,7 +287,10 @@ def simplify_graph(G, strict=True, remove_rings=True):
 
             # get edge between these nodes: if multiple edges exist between
             # them (see above), we retain only one in the simplified graph
-            edge_data = G.edges[u, v, 0]
+            # We can't assume that there exists an edge from u to v
+            # with key=0, so we get a list of all edges from u to v
+            # and just take the first one.
+            edge_data = list(G.get_edge_data(u, v).values())[0]
             for attr in edge_data:
                 if attr in path_attributes:
                     # if this key already exists in the dict, append it to the
@@ -283,6 +319,10 @@ def simplify_graph(G, strict=True, remove_rings=True):
             [Point((G.nodes[node]["x"], G.nodes[node]["y"])) for node in path]
         )
 
+        if track_merged:
+            # add the merged edges as a new attribute of the simplified edge
+            path_attributes["merged_edges"] = merged_edges
+
         # add the nodes and edge to their lists for processing at the end
         all_nodes_to_remove.extend(path[1:-1])
         all_edges_to_add.append(
@@ -298,14 +338,7 @@ def simplify_graph(G, strict=True, remove_rings=True):
     G.remove_nodes_from(set(all_nodes_to_remove))
 
     if remove_rings:
-        # remove any connected components that form a self-contained ring
-        # without any endpoints
-        wccs = nx.weakly_connected_components(G)
-        nodes_in_rings = set()
-        for wcc in wccs:
-            if not any(_is_endpoint(G, n) for n in wcc):
-                nodes_in_rings.update(wcc)
-        G.remove_nodes_from(nodes_in_rings)
+        G = _remove_rings(G)
 
     # mark graph as having been simplified
     G.graph["simplified"] = True
@@ -519,7 +552,6 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
     # regroup now that we potentially have new cluster labels from step 3
     groups = gdf.groupby("cluster")
     for cluster_label, nodes_subset in groups:
-
         osmids = nodes_subset.index.to_list()
         if len(osmids) == 1:
             # if cluster is a single node, add that node to new graph
@@ -565,11 +597,9 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
     # for every group of merged nodes with more than 1 node in it, extend the
     # edge geometries to reach the new node point
     for cluster_label, nodes_subset in groups:
-
         # but only if there were multiple nodes merged together,
         # otherwise it's the same old edge as in original graph
         if len(nodes_subset) > 1:
-
             # get coords of merged nodes point centroid to prepend or
             # append to the old edge geom's coords
             x = H.nodes[cluster_label]["x"]
