@@ -1,11 +1,10 @@
 """Graph utility functions."""
 
 import itertools
-import warnings
+from warnings import warn
 
 import geopandas as gpd
 import networkx as nx
-import numpy as np
 import pandas as pd
 from shapely.geometry import LineString
 from shapely.geometry import Point
@@ -42,7 +41,6 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
     crs = G.graph["crs"]
 
     if nodes:
-
         if not G.nodes:  # pragma: no cover
             raise ValueError("graph contains no nodes")
 
@@ -59,35 +57,32 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
         utils.log("Created nodes GeoDataFrame from graph")
 
     if edges:
-
         if not G.edges:  # pragma: no cover
             raise ValueError("graph contains no edges")
 
         u, v, k, data = zip(*G.edges(keys=True, data=True))
 
         if fill_edge_geometry:
-
             # subroutine to get geometry for every edge: if edge already has
             # geometry return it, otherwise create it using the incident nodes
             x_lookup = nx.get_node_attributes(G, "x")
             y_lookup = nx.get_node_attributes(G, "y")
 
-            def make_geom(u, v, data, x=x_lookup, y=y_lookup):
+            def _make_geom(u, v, data, x=x_lookup, y=y_lookup):
                 if "geometry" in data:
                     return data["geometry"]
                 else:
                     return LineString((Point((x[u], y[u])), Point((x[v], y[v]))))
 
-            geom = map(make_geom, u, v, data)
+            geom = map(_make_geom, u, v, data)
             gdf_edges = gpd.GeoDataFrame(data, crs=crs, geometry=list(geom))
 
         else:
             gdf_edges = gpd.GeoDataFrame(data)
             if "geometry" not in gdf_edges.columns:
                 # if no edges have a geometry attribute, create null column
-                gdf_edges["geometry"] = np.nan
-            gdf_edges.set_geometry("geometry")
-            gdf_edges.crs = crs
+                gdf_edges = gdf_edges.set_geometry([None] * len(gdf_edges))
+            gdf_edges = gdf_edges.set_crs(crs)
 
         # add u, v, key attributes as index
         gdf_edges["u"] = u
@@ -151,9 +146,10 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
         except (AssertionError, ValueError):  # pragma: no cover
             # AssertionError if x/y coords don't match geometry column
             # ValueError if geometry column contains non-point geometry types
-            warnings.warn(
+            warn(
                 "discarding the gdf_nodes geometry column, though its "
-                "values differ from the coordinates in the x and y columns"
+                "values differ from the coordinates in the x and y columns",
+                stacklevel=2,
             )
         gdf_nodes = gdf_nodes.drop(columns=gdf_nodes.geometry.name)
 
@@ -181,34 +177,60 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     return G
 
 
-def get_route_edge_attributes(
-    G, route, attribute=None, minimize_key="length", retrieve_default=None
-):
+def route_to_gdf(G, route, weight="length"):
     """
-    Get a list of attribute values for each edge in a path.
+    Return a GeoDataFrame of the edges in a path, in order.
 
     Parameters
     ----------
     G : networkx.MultiDiGraph
         input graph
     route : list
-        list of nodes IDs constituting the path
+        list of node IDs constituting the path
+    weight : string
+        if there are parallel edges between two nodes, choose lowest weight
+
+    Returns
+    -------
+    gdf_edges : geopandas.GeoDataFrame
+        GeoDataFrame of the edges
+    """
+    pairs = zip(route[:-1], route[1:])
+    uvk = ((u, v, min(G[u][v].items(), key=lambda i: i[1][weight])[0]) for u, v in pairs)
+    return graph_to_gdfs(G.subgraph(route), nodes=False).loc[uvk]
+
+
+def get_route_edge_attributes(
+    G, route, attribute=None, minimize_key="length", retrieve_default=None
+):
+    """
+    Do not use: deprecated.
+
+    Use the `route_to_gdf` function instead.
+
+    Parameters
+    ----------
+    G : networkx.MultiDiGraph
+        deprecated
+    route : list
+        deprecated
     attribute : string
-        the name of the attribute to get the value of for each edge. If None,
-        the complete data dict is returned for each edge.
+        deprecated
     minimize_key : string
-        if there are parallel edges between two nodes, select the one with the
-        lowest value of minimize_key
+        deprecated
     retrieve_default : Callable[Tuple[Any, Any], Any]
-        function called with the edge nodes as parameters to retrieve a
-        default value, if the edge does not contain the given attribute
-        (otherwise a `KeyError` is raised)
+        deprecated
 
     Returns
     -------
     attribute_values : list
-        list of edge attribute values
+        deprecated
     """
+    warn(
+        "The `get_route_edge_attributes` function has been deprecated and will "
+        "be removed in a future release. Use the `route_to_gdf` function instead.",
+        stacklevel=2,
+    )
     attribute_values = []
     for u, v in zip(route[:-1], route[1:]):
         # if there are parallel edges between two nodes, select the one with the
@@ -368,16 +390,12 @@ def get_undirected(G):
     # edge or self-loop. so, look through the edges and remove any duplicates.
     duplicate_edges = set()
     for u1, v1, key1, data1 in H.edges(keys=True, data=True):
-
         # if we haven't already flagged this edge as a duplicate
         if (u1, v1, key1) not in duplicate_edges:
-
             # look at every other edge between u and v, one at a time
             for key2 in H[u1][v1]:
-
                 # don't compare this edge to itself
                 if key1 != key2:
-
                     # compare the first edge's data to the second's
                     # if they match up, flag the duplicate for removal
                     data2 = H.edges[u1, v1, key2]
@@ -414,7 +432,6 @@ def _is_duplicate_edge(data1, data2):
 
     # if they contain the same osmid or set of osmids (due to simplification)
     if osmid1 == osmid2:
-
         # if both edges have geometry attributes and they match each other
         if ("geometry" in data1) and ("geometry" in data2):
             if _is_same_geometry(data1["geometry"], data2["geometry"]):
@@ -490,10 +507,8 @@ def _update_edge_keys(G):
 
     # for each group of duplicate edges
     for _, group in groups:
-
         # for each pair of edges within this group
         for geom1, geom2 in itertools.combinations(group["geometry"], 2):
-
             # if they don't have the same geometry, flag them as different
             # streets: flag edge uvk, but not edge vuk, otherwise we would
             # increment both their keys and they'll still duplicate each other
