@@ -1,6 +1,6 @@
-"""Unit tests for the package."""
+"""Test suite for the package."""
 
-# use agg backend so you don't need a display on travis-ci
+# use agg backend so you don't need a display on ci
 # do this first before pyplot is imported by anything
 import matplotlib as mpl
 
@@ -12,6 +12,7 @@ import os
 import tempfile
 from collections import OrderedDict
 from pathlib import Path
+from xml.etree import ElementTree as etree
 
 import folium
 import geopandas as gpd
@@ -29,7 +30,6 @@ from shapely.geometry import Polygon
 
 import osmnx as ox
 
-
 ox.config(log_console=True)
 ox.settings.log_console = True
 ox.settings.log_file = True
@@ -43,7 +43,7 @@ ox.settings.cache_folder = ".temp/cache"
 location_point = (37.791427, -122.410018)
 address = "600 Montgomery St, San Francisco, California, USA"
 place1 = {"city": "Piedmont", "state": "California", "country": "USA"}
-place2 = "Civic Center, Los Angeles, California"
+place2 = "SoHo, New York, NY"
 p = (
     "POLYGON ((-122.262 37.869, -122.255 37.869, -122.255 37.874,"
     "-122.262 37.874, -122.262 37.869))"
@@ -112,8 +112,10 @@ def test_stats():
     G.add_node(0, x=location_point[1], y=location_point[0])
     _ = ox.bearing.calculate_bearing(0, 0, 1, 1)
     G = ox.add_edge_bearings(G)
+    G = ox.add_edge_bearings(G, precision=2)
     G_proj = ox.project_graph(G)
     G_proj = ox.distance.add_edge_lengths(G_proj, edges=tuple(G_proj.edges)[0:3])
+    G_proj = ox.distance.add_edge_lengths(G_proj, edges=tuple(G_proj.edges)[0:3], precision=2)
 
     # calculate stats
     cspn = ox.stats.count_streets_per_node(G)
@@ -125,7 +127,8 @@ def test_stats():
     Gu = ox.get_undirected(G)
     entropy = ox.bearing.orientation_entropy(Gu, weight="length")
     fig, ax = ox.bearing.plot_orientation(Gu, area=True, title="Title")
-    fig, ax = ox.bearing.plot_orientation(Gu, ax=ax, area=False, title="Title")
+    fig, ax = ox.plot_orientation(Gu, area=True, title="Title")
+    fig, ax = ox.plot_orientation(Gu, ax=ax, area=False, title="Title")
 
     # test cleaning and rebuilding graph
     G_clean = ox.consolidate_intersections(G_proj, tolerance=10, rebuild_graph=True, dead_ends=True)
@@ -173,18 +176,31 @@ def test_osm_xml():
 
     # test osm xml output from gdfs
     nodes, edges = ox.graph_to_gdfs(G)
-    ox.save_graph_xml([nodes, edges])
+    ox.osm_xml.save_graph_xml([nodes, edges])
 
     # test ordered nodes from way
     df = pd.DataFrame({"u": [54, 2, 5, 3, 10, 19, 20], "v": [76, 3, 8, 10, 5, 20, 15]})
     ordered_nodes = ox.osm_xml._get_unique_nodes_ordered_from_way(df)
     assert ordered_nodes == [2, 3, 10, 5, 8]
 
+    # test roundabout handling
+    default_overpass_settings = ox.settings.overpass_settings
+    ox.settings.overpass_settings += '[date:"2023-04-01T00:00:00Z"]'
+    point = (39.0290346, -84.4696884)
+    G = ox.graph_from_point(point, dist=500, dist_type="bbox", network_type="drive", simplify=False)
+    gdf_edges = ox.graph_to_gdfs(G, nodes=False)
+    gdf_way = gdf_edges[gdf_edges["osmid"] == 570883705]  # roundabout
+    first = gdf_way.iloc[0].dropna().astype(str)
+    root = etree.Element("osm", attrib={"version": "0.6", "generator": "OSMnx"})
+    edge = etree.SubElement(root, "way")
+    ox.osm_xml._append_nodes_as_edge_attrs(edge, first, gdf_way)
+
+    # restore settings
+    ox.settings.overpass_settings = default_overpass_settings
     ox.settings.all_oneway = default_all_oneway
 
 
 def test_elevation():
-
     G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
     rasters = list(Path("tests/input_data").glob("elevation*.tif"))
 
@@ -197,16 +213,27 @@ def test_elevation():
 
     # add edge grades and their absolute values
     G = ox.add_edge_grades(G, add_absolute=True)
+    G = ox.add_edge_grades(G, add_absolute=True, precision=2)
 
 
 def test_routing():
-
     G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
 
     # give each edge speed and travel time attributes
     G = ox.add_edge_speeds(G)
-    G = ox.add_edge_speeds(G, hwy_speeds={"motorway": 100})
+    G = ox.add_edge_speeds(G, hwy_speeds={"motorway": 100}, precision=2)
     G = ox.add_edge_travel_times(G)
+    G = ox.add_edge_travel_times(G, precision=2)
+
+    # test value cleaning
+    assert ox.speed._clean_maxspeed("100,2") == 100.2
+    assert ox.speed._clean_maxspeed("100.2") == 100.2
+    assert ox.speed._clean_maxspeed("100 km/h") == 100.0
+    assert ox.speed._clean_maxspeed("100 mph") == pytest.approx(160.934)
+    assert ox.speed._clean_maxspeed("60|100") == 80
+    assert ox.speed._clean_maxspeed("60|100 mph") == pytest.approx(128.7472)
+    assert ox.speed._clean_maxspeed("signal") is None
+    assert ox.speed._clean_maxspeed("100;70") is None
 
     orig_x = np.array([-122.404771])
     dest_x = np.array([-122.401429])
@@ -215,7 +242,18 @@ def test_routing():
     orig_node = ox.distance.nearest_nodes(G, orig_x, orig_y)[0]
     dest_node = ox.distance.nearest_nodes(G, dest_x, dest_y)[0]
 
+    # test non-numeric weight (should raise ValueError)
+    try:
+        route = ox.shortest_path(G, orig_node, dest_node, weight="highway")
+        raise AssertionError()
+    except ValueError:
+        pass
+    # test missing weight (should raise warning)
+    route = ox.shortest_path(G, orig_node, dest_node, weight="time")
+    # test good weight
     route = ox.shortest_path(G, orig_node, dest_node, weight="travel_time")
+
+    route_edges = ox.utils_graph.route_to_gdf(G, route, "travel_time")
     attributes = ox.utils_graph.get_route_edge_attributes(G, route)
     attributes = ox.utils_graph.get_route_edge_attributes(G, route, "travel_time")
 
@@ -285,7 +323,6 @@ def test_plots():
 
 
 def test_find_nearest():
-
     # get graph and x/y coords to search
     G = ox.graph_from_point(location_point, dist=500, network_type="drive", simplify=False)
     Gp = ox.project_graph(G)
@@ -303,10 +340,16 @@ def test_find_nearest():
     ne2 = ox.distance.nearest_edges(G, X[0], Y[0], interpolate=50, return_dist=True)
 
 
-def test_api_endpoints():
+def test_endpoints():
+    ip = ox._downloader._resolve_host_via_doh("overpass-api.de")
+    ip = ox._downloader._resolve_host_via_doh("AAAAAAAAAAA")
 
-    ip = ox.downloader._get_host_by_name("overpass-api.de")
-    ip = ox.downloader._get_host_by_name("AAAAAAAAAAA")
+    _doh_url_template_default = ox.settings.doh_url_template
+    ox.settings.doh_url_template = "http://aaaaaa.hostdoesntexist.org/nothinguseful"
+    ip = ox._downloader._resolve_host_via_doh("overpass-api.de")
+    ox.settings.doh_url_template = None
+    ip = ox._downloader._resolve_host_via_doh("overpass-api.de")
+    ox.settings.doh_url_template = _doh_url_template_default
 
     params = OrderedDict()
     params["format"] = "json"
@@ -314,11 +357,11 @@ def test_api_endpoints():
 
     # Bad Address - should return an empty response
     params["q"] = "AAAAAAAAAAA"
-    response_json = ox.downloader.nominatim_request(params=params, request_type="search")
+    response_json = ox._downloader._nominatim_request(params=params, request_type="search")
 
     # Good Address - should return a valid response with a valid osm_id
     params["q"] = "Newcastle A186 Westgate Rd"
-    response_json = ox.downloader.nominatim_request(params=params, request_type="search")
+    response_json = ox._downloader._nominatim_request(params=params, request_type="search")
 
     # Lookup
     params = OrderedDict()
@@ -326,11 +369,11 @@ def test_api_endpoints():
     params["address_details"] = 0
     params["osm_ids"] = "W68876073"
 
-    response_json = ox.downloader.nominatim_request(params=params, request_type="lookup")
+    response_json = ox._downloader._nominatim_request(params=params, request_type="lookup")
 
     # Invalid nominatim query type
     with pytest.raises(ValueError):
-        response_json = ox.downloader.nominatim_request(params=params, request_type="xyz")
+        response_json = ox._downloader._nominatim_request(params=params, request_type="xyz")
 
     default_key = ox.settings.nominatim_key
     default_nominatim_endpoint = ox.settings.nominatim_endpoint
@@ -338,18 +381,12 @@ def test_api_endpoints():
 
     # Searching on public nominatim should work even if a key was provided
     ox.settings.nominatim_key = "NOT_A_KEY"
-    response_json = ox.downloader.nominatim_request(params=params, request_type="search")
-
-    # Test changing the endpoint.
-    # It should fail because we didn't provide a valid key
-    ox.settings.nominatim_endpoint = "http://open.mapquestapi.com/nominatim/v1/"
-    with pytest.raises(Exception):
-        response_json = ox.downloader.nominatim_request(params=params, request_type="search")
+    response_json = ox._downloader._nominatim_request(params=params, request_type="search")
 
     # Test changing the endpoint.
     # This should fail because we didn't provide a valid endpoint
     ox.settings.overpass_endpoint = "http://NOT_A_VALID_ENDPOINT/api/"
-    with pytest.raises(Exception):
+    with pytest.raises(Exception) as ex:
         G = ox.graph_from_place(place1, network_type="all")
 
     ox.settings.nominatim_key = default_key
@@ -358,7 +395,6 @@ def test_api_endpoints():
 
 
 def test_graph_save_load():
-
     # save graph as shapefile and geopackage
     G = ox.graph_from_point(location_point, dist=500, network_type="drive")
     ox.save_graph_shapefile(G, directed=True)
@@ -426,7 +462,6 @@ def test_graph_save_load():
 
 
 def test_graph_from_functions():
-
     # graph from bounding box
     _ = ox.utils_geo.bbox_from_point(location_point, project_utm=True, return_crs=True)
     north, south, east, west = ox.utils_geo.bbox_from_point(location_point, dist=500)
@@ -447,7 +482,8 @@ def test_graph_from_functions():
     G = ox.graph_from_place([place1], network_type="all", clean_periphery=False)
 
     # graph from polygon
-    G = ox.graph_from_polygon(polygon, network_type="walk", truncate_by_edge=True)
+    G = ox.graph_from_polygon(polygon, network_type="walk", truncate_by_edge=True, simplify=False)
+    G = ox.simplify_graph(G, strict=False, remove_rings=False, track_merged=True)
 
     # test custom query filter
     cf = (
@@ -471,10 +507,12 @@ def test_graph_from_functions():
     )
 
 
-def test_geometries():
-
-    # geometries_from_bbox - bounding box query to return empty GeoDataFrame
-    gdf = ox.geometries_from_bbox(0.009, -0.009, 0.009, -0.009, tags={"building": True})
+def test_features():
+    # geometries_from_bbox - bounding box query to return no data
+    try:
+        gdf = ox.geometries_from_bbox(0.009, -0.009, 0.009, -0.009, tags={"building": True})
+    except ox._errors.EmptyOverpassResponse:
+        pass
 
     # geometries_from_bbox - successful
     north, south, east, west = ox.utils_geo.bbox_from_point(location_point, dist=500)
@@ -491,6 +529,10 @@ def test_geometries():
     gdf = ox.geometries_from_place(place1, tags=tags)
     gdf = ox.geometries_from_place([place1], tags=tags)
 
+    # geometries_from_polygon
+    polygon = ox.geocode_to_gdf(place1).geometry.iloc[0]
+    ox.geometries_from_polygon(polygon, tags)
+
     # geometries_from_address - includes testing overpass settings and snapshot from 2019
     ox.settings.overpass_settings = '[out:json][timeout:200][date:"2019-10-28T19:20:00Z"]'
     gdf = ox.geometries_from_address(address, tags=tags)
